"""build command."""

import click

from pyrgo.utilities.command import PythonExecCommand, inform_and_run_program


@click.command
def build() -> None:
    """Build project."""
    inform_and_run_program(
        command=PythonExecCommand(program="hatch").add_args(
            args=[
                "build",
            ],
        ),
    )



"""docs build documentation."""
import sys
from typing import List

if sys.version_info >= (3, 9):
    from typing import Literal
else:
    from typing_extensions import Literal

import click

from pyrgo.utilities.command import PythonExecCommand, inform_and_run_program


@click.command()
@click.option(
    "-t",
    "--theme",
    "theme",
    type=click.Choice(choices=["material", "mkdocs", "readthedocs"]),
    default="material",
    help="The theme to use when building your documentation.",
)
@click.option(
    "-s",
    "--strict",
    "strict",
    is_flag=True,
    default=False,
    help="Enable strict mode. This will cause MkDocs to abort the build on any warnings.",  # noqa: E501
)
def build(
    *,
    theme: Literal[
        "material",
        "mkdocs",
        "readthedocs",
    ],
    strict: bool,
) -> None:
    """Build project documentation."""
    args_to_add: List[str] = ["build"]
    if strict:
        args_to_add.append("--strict")
    args_to_add.extend(["--theme", theme])
    inform_and_run_program(
        command=PythonExecCommand(
            program="mkdocs",
        ).add_args(
            args=args_to_add,
        ),
    )



"""test command."""

import pathlib
from typing import List, Optional

import click

from pyrgo.utilities.command import PythonExecCommand, inform_and_run_program
from pyrgo.utilities.project import (
    list_pytest_markers,
    read_pyproject,
)


def dynamic_marker_choices() -> List[str]:
    """Dynamic markers for options."""
    cwd = pathlib.Path().cwd()
    content = read_pyproject(cwd=cwd)
    return list_pytest_markers(content=content)


@click.command()
@click.option(
    "-m",
    "--marked",
    "marker",
    type=click.Choice(
        choices=dynamic_marker_choices(),
    ),
    default=None,
    help="Run tests based on marks. By default all tests are executed.",
)
@click.option(
    "--no-strict-markers",
    "no_strict_markers",
    type=bool,
    is_flag=True,
    default=False,
    help="Any unknown marks applied with the `@pytest.mark.name_of_mark` decorator will trigger an error.",  # noqa: E501
)
def test(
    *,
    no_strict_markers: bool,
    marker: Optional[str],
) -> None:
    """Execute tests using `pytest`."""
    args_to_add: List[str] = []
    if not no_strict_markers:
        args_to_add.append("--strict-markers")

    if marker:
        args_to_add.extend(["-m", marker])

    inform_and_run_program(
        command=PythonExecCommand(program="pytest").add_args(
            args=args_to_add,
        ),
    )



"""command utilities."""
import subprocess
import sys
from typing import List

if sys.version_info >= (3, 9):
    from typing import Literal
else:
    from typing_extensions import Literal


import click
from typing_extensions import Self


class PythonExecCommand:
    """Command builder."""

    def __init__(self, program: str) -> None:
        """Python programs to be executed within current python interpreter."""
        self.program = program
        self.args = [sys.executable, "-m", program]

    def add_args(self, args: List[str]) -> Self:
        """Add args to execution."""
        self.args.extend(args)
        return self

    def run(self) -> None:
        """Run command."""
        subprocess.run(args=self.args)


def colorize_text(
    *,
    text: str,
    color: Literal[
        "yellow",
        "red",
    ],
) -> str:
    """Colorize text."""
    return click.style(text=text, fg=color)


def inform_and_run_program(command: PythonExecCommand) -> None:
    """Inform and run program."""
    click.echo(
        message=colorize_text(
            text=f"running {command.program}...",
            color="yellow",
        ),
    )
    command.run()



"""docs new command."""

import pathlib
import sys

import click

from pyrgo.utilities.command import (
    PythonExecCommand,
    colorize_text,
    inform_and_run_program,
)


@click.command()
def new() -> None:
    """Create a new MkDocs project."""
    cwd = pathlib.Path().cwd()
    pyproject = cwd.joinpath("pyproject.toml")
    if not pyproject.exists():
        click.echo(
            message=colorize_text(
                text="pyproject.toml not found.",
                color="red",
            ),
        )
        sys.exit(1)

    inform_and_run_program(
        command=PythonExecCommand(program="mkdocs").add_args(
            [
                "new",
                str(cwd),
            ],
        ),
    )



"""Cli utilitites."""
from typing import List

import click


def add_commands(
    cli: click.Group,
    commands: List[click.Command],
) -> None:
    """Add commands to CLI group."""
    for command in commands:
        cli.add_command(cmd=command)



"""project utilities."""
import pathlib
from typing import Any, Dict, List

import tomli


def read_pyproject(cwd: pathlib.Path) -> Dict[str, Any]:
    """Read `pyproject.toml`."""
    path_to_pyproject = cwd.joinpath("pyproject.toml")
    if not path_to_pyproject.exists():
        raise RuntimeError

    return tomli.loads(path_to_pyproject.read_text(encoding="utf-8"))


def extract_pytest_relevant_paths(content: Dict[str, Any]) -> List[str]:
    """Get relevant pytest paths from `pyproject.toml` content."""
    return content["tool"]["pytest"]["ini_options"]["testpaths"]


def extract_relevent_paths(content: Dict[str, Any]) -> List[str]:
    """Get relevant project paths from `pyproject.toml` content."""
    relevant_paths = extract_pytest_relevant_paths(content=content)
    relevant_paths.append(content["project"]["name"])
    return relevant_paths


def list_pytest_markers(content: Dict[str, Any]) -> List[str]:
    """List pytest markers."""

    def _split_and_first_element(x: str) -> str:
        return x.split(":")[0]

    return [
        _split_and_first_element(x=x)
        for x in content["tool"]["pytest"]["ini_options"]["markers"]
    ]



"""fmt command."""

import pathlib

import click

from pyrgo.utilities.command import (
    PythonExecCommand,
    inform_and_run_program,
)
from pyrgo.utilities.project import extract_relevent_paths, read_pyproject


@click.command()
def fmt() -> None:
    """Format all files of the current project using `black` and `ruff`."""
    cwd = pathlib.Path().cwd()
    content = read_pyproject(cwd=cwd)
    relevant_paths = extract_relevent_paths(content=content)

    inform_and_run_program(
        command=PythonExecCommand(program="ruff").add_args(
            args=["--fix-only", *relevant_paths],
        ),
    )

    inform_and_run_program(
        command=PythonExecCommand(program="black").add_args(
            args=relevant_paths,
        ),
    )



"""test command utilitites."""
import sys

if sys.version_info >= (3, 9):
    from typing import Literal
else:
    from typing_extensions import Literal

import pytest

from pyrgo.utilities.command import colorize_text


@pytest.mark.unit()
@pytest.mark.parametrize(
    argnames=["color", "expected_text"],
    argvalues=[
        (
            "yellow",
            "\x1b[33mhi\x1b[0m",
        ),
        (
            "red",
            "\x1b[31mhi\x1b[0m",
        ),
    ],
)
def test_colorize_text(color: Literal["yellow", "red"], expected_text: str) -> None:
    """Test colorize text works as expected."""
    colorized_text = colorize_text(text="hi", color=color)
    assert colorized_text == expected_text  # noqa: S101



"""Root CLI."""

import click

from pyrgo.command.add import add
from pyrgo.command.build import build
from pyrgo.command.check import check
from pyrgo.command.clean import clean
from pyrgo.command.docs import docs
from pyrgo.command.fmt import fmt
from pyrgo.command.init import init
from pyrgo.command.lock import lock
from pyrgo.command.new import new
from pyrgo.command.remove import remove
from pyrgo.command.test import test
from pyrgo.command.venv import venv
from pyrgo.utils import add_commands


@click.group()
@click.version_option()
def cli() -> None:
    """pyrgo. Python package manager."""


add_commands(
    cli=cli,
    commands=[
        check,
        clean,
        fmt,
        init,
        lock,
        new,
        venv,
        add,
        test,
        docs,
        remove,
        build,
    ],
)



"""check command."""


import pathlib

import click

from pyrgo.utilities.command import PythonExecCommand, inform_and_run_program
from pyrgo.utilities.project import extract_relevent_paths, read_pyproject


@click.command()
def check() -> None:
    """Analyze the current package with `ruff` and `mypy`."""
    cwd = pathlib.Path().cwd()
    content = read_pyproject(cwd=cwd)
    relevant_paths = extract_relevent_paths(content=content)
    inform_and_run_program(
        command=PythonExecCommand(program="ruff").add_args(
            args=relevant_paths,
        ),
    )
    inform_and_run_program(
        command=PythonExecCommand(program="mypy").add_args(
            args=relevant_paths,
        ),
    )
